#!/usr/bin/python
# -*- coding: UTF-8 -*-

import tkinter as tk
from light import Light
from roomdialog import RoomDialog
from rooms import Room
from switch import Switch
from daligateway import DaliGateWay
from open_comport import open_comport
from readxml import xml_read


# XMLFILE = "Beispiel_small.xml"
XMLFILE = "Beispiel_large.xml"


class Floor(object):
    def __init__(self, ser):
        self.floor = tk.Tk()
        # hier werden die Projektdaten aus dem XML-File eingelesen
        self.floorfile, self.floorplansize, self.floorwindowsize, projektname, self.roomlist, \
            self.lightlist, self.switchlist = xml_read(XMLFILE)

        self.floor.title(projektname + " @ {}:".format(ser.name))
        self.floor.resizable(width=False, height=False)
        self.floor.protocol('WM_DELETE_WINDOW', self.quit)
        self.dali = DaliGateWay(ser)

        # Referenzen auf das DaliGateway verteilen
        Light.dali = self.dali
        Room.dali = self.dali

        self.rooms = []
        self.add_rooms()
        self.canvas = None
        self.add_floorplan()

    def quit(self):
        print("App.quit, stopping Thread RS232")
        self.dali.quit()
        self.floor.destroy()

    def add_rooms(self):
        # Koordinaten-Rechtecke zur Beschreibung der Raumfläche,
        # x, y = linke obere Ecke - Breite, Höhe
        # jede (Teil-)Fläche wird als Tuple definiert,
        # es können mehrere Rechtecke definiert werden (als Tuple aus Tuplen), Raumbezeichnung
        rooms = self.roomlist

        idx = 0
        for room in rooms:
            # Die Raumflächendefinition kann aus mehreren Rechtecken bestehen
            rectangles = []
            for rectangle in room[0]:
                rectangles.append(
                    (rectangle[0], rectangle[1],
                     rectangle[0] + rectangle[2],
                     rectangle[1] + rectangle[3]))
            self.rooms.append(Room(self.floor, tuple(rectangles), idx, light_group_id=room[2], name=room[1]))
            idx += 1

    def add_floorplan(self):
        self.canvas = tk.Canvas(self.floor, width=self.floorwindowsize[0], height=self.floorwindowsize[1],
                                scrollregion=(0, 0, self.floorplansize[0], self.floorplansize[1]))
        background = tk.PhotoImage(file=self.floorfile)
        self.canvas.create_image(0, 0, anchor=tk.NW, image=background)
        self.canvas.bind('<3>', self.floor_is_clicked_3)
        self.canvas.grid(column=0, row=0, padx=10, pady=10, sticky="nsew")

        # Horizontale Scrollbars aktivieren, wenn das Window kleiner als die Grafik ist
        if self.floorwindowsize[1] < self.floorplansize[1]:
            yscrollbar = tk.Scrollbar(self.floor, orient=tk.VERTICAL, command=self.canvas.yview, width=20)
            yscrollbar.grid(column=1, row=0, rowspan=1, pady=5, sticky="nsew")
            self.canvas.configure(yscrollcommand=yscrollbar.set)

        if self.floorwindowsize[0] < self.floorplansize[0]:
            xscrollbar = tk.Scrollbar(self.floor, orient=tk.HORIZONTAL, command=self.canvas.xview, width=20)
            xscrollbar.grid(column=0, columnspan=1, row=1, padx=5, sticky="nsew")
            self.canvas.configure(xscrollcommand=xscrollbar.set)

        for lamp in self.lightlist:
            self.rooms[lamp[2]].add_lamp(Light(self.floor, self.canvas, x=lamp[0], y=lamp[1], shape=lamp[5],
                                               length=lamp[6], room=self.rooms[lamp[2]], is_dimmable=lamp[3],
                                               name=lamp[4], group=lamp[7], dali_id=lamp[8]))

        for switch in self.switchlist:
            self.rooms[switch[2]].add_switch(Switch(self.canvas, switch[0], switch[1], room=self.rooms[switch[2]],
                                                    group=switch[3], callback=self.switch_is_clicked))
        self.floor.mainloop()

    def floor_is_clicked_3(self, e):
        """
        Ein Mausclick mit der Rechten Maustaste ruft diese Funktion auf
        :param e:
        :return:
        """
        # Herausfinden, in welchem Raum geklickt wurde.
        # Dazu nacheinander alle Räume abfragen, ob die Mauskoordinaten innerhalb ihrer Raumgrenzen liegen
        for room in self.rooms:
            # Dazu die Koordinaten auf den Ursprung des Canvas beziehen, nicht auf das aktuelle Widget,
            # das möglicherweise durch Scrollbars verschoben ist.
            r = room.is_clicked(self.canvas.canvasx(e.x), self.canvas.canvasy(e.y))
            # wenn ein Raum gefunden wurde, dann wird sein Index in der Raumliste zurückgeliefert
            # oder -1, wenn der Mausclick außerhalb von definierten Raumflächen lag.
            if r >= 0:
                dialog = RoomDialog(self.floor, self.rooms[r].get_name())
                level, scene = dialog.start()
                # liefert ein Light-Level oder eine Scene zurück
                # als Tuple level, scene
                # Abbruch = (-1, -1)
                # Level = (level, -1)
                # Scene = (-1, scene)
                if level >= 0 and scene == -1:
                    self.rooms[r].set_lamps(level)
                elif scene >= 0:
                    self.rooms[r].recall_scenes(scene)
                # keine weiteren Räume mehr untersuchen
                break
        # Wenn der Click außerhalb aller Raumpolygone lag, dann ein akustisches Zeichen ausgeben
        if r < 0:
            self.canvas.bell()

    def switch_is_clicked(self, group, state):
        for room in self.rooms:
            room.set_group_level(group, state)


if __name__ == '__main__':
    # An dieser Stelle kann der COM-Port festgelegt werden, der gesucht und  geöffnet werden soll
    # als Parameter: "COM10"
    Floor(open_comport())
